
var BaseProp = new Class({
  /* 
  Construct a base property consisting of the property name and its value.
  */
  initialize: function(name, value) {
    this.className = "BaseProp";
    this.name = name;
    this.value = value;
    this.setParams = 1;
    this.hidden = false;
  },

  /*
  Extract the saved property value from xml and process the string.
  */
  load: function(xml) {
    this.set(String(xml.evaluate('string(property[@name="' + 
                                 this.name + '"]/text())')));
  },

  /*
  Save the property value as xml item.
  */
  save: function() {
    return '<property name="' + this.name + '">' + this.value + '</property>';
  },

  /*
  Create one or two FormField items and push them onto the ff array.
  */
  getFormField: function(ff) {
  },
  
  /*
  Check the property value and do conversions if necessary.
  */
  set: function(value) {
    if (value != null) {
      this.value = value;
    }
  },
  
  /**
   * Reset value to defaultValue
   */
  reset: function(value) {
    if (this.defaultValue) {
      if (value) {
        this.defaultValue = value;
      }
      this.value = this.defaultValue;
    }
  }
});

/* ------------------------------------------------------------------------ */

var StringProp = BaseProp.extend({
  initialize: function(name, value) {
    this.parent(name, String(value));
    this.className = "StringProp";
  },

  escapeXML: function(value) {
    var v = (value)?value:"";
    try {
      v = v.replace(/&/g, "&amp;");
      v = v.replace(/</g, "&lt;");
      v = v.replace(/>/g, "&gt;");
      v = v.replace(/'/g, "&apos;");
      v = v.replace(/"/g, "&quot;");
    } catch (e) {
      print("StringProp.escapeXML("+v+")");
    }
    return v;
  },

  save: function() {
    return '<property name="' + this.name + '">' + 
           this.escapeXML(this.value) + '</property>';
  }
});

/* ------------------------------------------------------------------------ */

var BoolProp = BaseProp.extend({
  initialize: function(name, title, defaultValue, description) {
    this.parent(name, defaultValue);
    this.className = "BoolProp";
    this.title = tr(title);
    this.defaultValue = defaultValue;
    this.description = description ? tr(description) : null;
  },

  load: function(xml) {
    this.set(String(xml.evaluate('string(property[@name="' + this.name + 
                                 '"]/text())')) == "yes" ? 1 : 0);
  },

  save: function() {
    return '<property name="' + this.name + '">' + 
           (this.value ? "yes" : "no") + '</property>';
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "checkbox";
    f.value = this.value ? 1 : 0;
    f.defaultValue = this.defaultValue;
    f.description = this.description;
    ff.push(f);
  },
  
  set: function(value) {
    this.value = (value == 1);
  }
});

/* ------------------------------------------------------------------------ */

var TextProp = StringProp.extend({
  initialize: function(name, title, defaultValue, description, secure) {
    this.parent(name, defaultValue);
    this.className = "TextProp";
    this.title = tr(title);
    this.description = description ? tr(description) : null;
    this.secure = (secure == true);
    this.defaultValue = defaultValue;
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "text";
    f.value = this.value;
    f.description = this.description;
    f.secure = this.secure;
    ff.push(f);
  },
  
  set: function(value) {
    this.value = value ? value : this.defaultValue;
  }
});

/* ------------------------------------------------------------------------ */

var ListProp = BaseProp.extend({
  initialize: function(name, title, defaultValue, optTexts, optValues, 
                        description) {
    this.parent(name, defaultValue);
    this.className = "ListProp";
    this.title = tr(title);
    this.defaultValue = defaultValue;
    this.optTexts = optTexts;
    this.optValues = optValues;
    this.description = description ? tr(description) : null;
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "popup";
    f.option = this.optTexts;
    f.optionValue = this.optValues;
    f.defaultValue = this.defaultValue;
    f.value = this.value;
    f.description = this.description;
    ff.push(f);
  },
  
  set: function(value) {
    value = parseInt(value);
    this.value = !isNaN(value) ? value : this.defaultValue;
  }
});

/* ------------------------------------------------------------------------ */

var IntervalProp = BaseProp.extend({
  initialize: function(value, minValue) {
    this.parent("interval", value);
    this.className = "IntervalProp";
    this.minValue = minValue;
    switch (minValue) {
      case  1: this.maxValue =  10; break;
      case  3: this.maxValue =  60; break;
      case 10: this.maxValue = 120; break;
      default: this.maxValue = 300; break;
    }
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = tr("Update interval in sec.:");
    f.type = "slider";
    f.minLength = this.minValue;
    f.maxLength = this.maxValue;
    //f.ticks = (f.minLength < 10 ? 30 : Math.floor(f.maxLength / f.minLength)) - 1;
    f.tickLabel = [this.minValue + tr(" sec."), 
                   Math.floor(300 / (f.ticks + 1)) + tr(" sec./tick"), 
                   "5" + tr(" min.")];
    f.defaultValue = this.value;
    ff.push(f);
  },
  
  set: function(value) {
    this.value = parseFloat(value);
    if (isNaN(this.value) || (this.value < this.minValue)) {
      this.value = this.minValue;
    }
    if (this.value > this.maxValue) this.value = this.maxValue;
  }
});

/* ------------------------------------------------------------------------ */

var Interval2Prop = ListProp.extend({
  initialize: function(value, minValue) {
    var optValues = [1 * 60, 5 * 60, 10 * 60, 15 * 60, 20 * 60, 30 * 60, 
                     45 * 60, 60 * 60, 120 * 60, 360 * 60, 720 * 60];
    var optTexts = ["1 " + tr("min."), "5 " + tr("min."), "10 " + tr("min."), 
                    "15 " + tr("min."), "20 " + tr("min."), "30 " + tr("min."), 
                    "45 " + tr("min."), "1 " + tr("hrs."), "2 " + tr("hrs."), 
                    "6 " + tr("hrs."), "12 " + tr("hrs.")];
    this.parent("interval", "Update interval:", 15 * 60, optTexts, optValues);
    this.className = "Interval2Prop";
  }
});

/* ------------------------------------------------------------------------ */

var ColorizeProp = BaseProp.extend({
  initialize: function(name, title, defaultValue) {
    if (defaultValue == null) {
      defaultValue = "#000000";
    }
    this.parent(name, defaultValue);
    this.className = "ColorizeProp";
    this.setParams = 2;
    this.title = tr(title);
    this.defaultValue = defaultValue;
    this.isActive = false;
    this.isDynamic = false;
  },
  
  load: function(xml) {
    var value = String(xml.evaluate('string(property[@name="' 
                                    + this.name + '"]/text())'));
    if (/...:/.test(value)) {
      this.set(/^dyn:/.test(value) ? 2 : (/^col:/.test(value) ? 1 : 0), 
               value.match(/#[0-9A-Fa-f]{6}/));
    } else {
      // for backwards compatibility
      this.set(/#[0-9A-Fa-f]{6}/.test(value) ? 1 : 0, value);
    }
  },

  save: function() {
    return '<property name="' + this.name + '">' 
         + (this.isDynamic ? "dyn:" : (this.isActive ? "col:" : "off:")) 
         + this.value + '</property>';
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "popup";
    f.option = [tr("Standard color"), tr("Following color selection")];
    f.optionValue = [0, 1];
    f.defaultValue = 0;
    f.value = this.isDynamic ? 2 : (this.isActive ? 1 : 0);
    ff.push(f);

    var f = new FormField();
    f.title = "";
    f.type = "color";
    f.value = this.value;
    ff.push(f);
  },
  
  set: function(value1, value2) {
    this.isDynamic = (value1 == 2);
    this.isActive = (value1 == 1);
    this.value = value2 ? value2 : this.defaultValue;
  },
  
  colorize: function(value) {
    return this.isActive ? this.value : "";
  }
});

/* ------------------------------------------------------------------------ */

var coDefault = 0;
var coActive = 1;
var coDynamic = 2;

var MultiColorProp = ColorizeProp.extend({
  initialize: function(name, title, defaultValue, defaultOption) {
    this.parent(name, title, defaultValue);
    this.className = "MultiColorProp";
    if (defaultOption) {
      switch (defaultOption) {
        case coDefault:
          break;
        case coActive:
          this.isActive = true;
          break;
        case coDynamic:
          this.isDynamic = true;
          break;
      }
    }
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "popup";
    f.option = [tr("Standard color"), tr("Following color selection"), 
                tr("Value dependent color (from green to red)")];
    f.optionValue = [0, 1, 2];
    f.defaultValue = 0;
    f.value = this.isDynamic ? 2 : (this.isActive ? 1 : 0);
    ff.push(f);

    var f = new FormField();
    f.title = "";
    f.type = "color";
    f.value = this.value;
    ff.push(f);
  },
  
  /* value is in percent [0.0 .. 1.0] */
  colorize: function(value) {
    if (this.isDynamic) {
      if (value < 0.50) {
        var r = Math.round(value * 510);
        return "#" + r.toString(16).pad(2, "0") + "FF00";
      }
      var g = Math.round((1.00 - value) * 510);
      return "#FF" + g.toString(16).pad(2, "0") + "00";
    }
    return this.isActive ? this.value : "";
  }
});

/* ------------------------------------------------------------------------ */

var OpacityProp = BaseProp.extend({
  initialize: function(name, title, defaultValue, minValue) {
    this.parent(name, defaultValue);
    this.className = "OpacityProp";
    this.title = tr(title);
    this.defaultValue = defaultValue;
    this.minValue = (minValue == null) ? 0 : minValue;
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "slider";
    f.ticks = 15;
    f.minLength = this.minValue;
    f.maxLength = 255;
    f.tickLabel = [Math.round((this.minValue * 100) / 255) + "%", tr("Opacity"), "100%"];
    f.value = this.value;
    ff.push(f);
  },
  
  set: function(value) {
    value = isNaN(value) ? this.defaultValue : parseInt(value);
    this.value = value < this.minValue ? this.minValue : value;
  }
});

/* ------------------------------------------------------------------------ */

var FileProp = StringProp.extend({
  initialize: function(name, title, description, extension) {
    this.parent(name, "");
    this.className = "FileProp";
    this.title = tr(title);
    this.description = description ? tr(description) : null;
    this.extension = extension;
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "selector";
    f.kind = "files";
    f.style = "open";
    f.value = this.value;
    f.description = this.description;
    f.extension = this.extension;
    ff.push(f);
  },
  
  set: function(value) {
    this.value = (value && filesystem.itemExists(value)) ? value : "";
  }
});

/* ------------------------------------------------------------------------ */

var FolderProp = StringProp.extend({
  initialize: function(name, title, description) {
    this.parent(name, "");
    this.className = "FolderProp";
    this.title = tr(title);
    this.description = description ? tr(description) : null;
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = this.title;
    f.type = "selector";
    f.kind = "folders";
    f.style = "open";
    f.value = this.value;
    f.description = this.description;
    ff.push(f);
  },
  
  set: function(value) {
    this.value = (value && filesystem.itemExists(value)) ? value : "";
  }
});

/* ------------------------------------------------------------------------ */

var RowLimitProp = ListProp.extend({
  initialize: function(title, description) {
    var optTexts = [tr("No limit"), 
                    "5 " + tr("messages max"), "10 " + tr("messages max"), 
                    "15 " + tr("messages max"), "20 " + tr("messages max"), 
                    "25 " + tr("messages max"), "30 " + tr("messages max"), 
                    "40 " + tr("messages max"), "50 " + tr("messages max")];
    var optValues = [-1, 5, 10, 15, 20, 25, 30, 40, 50];
    this.parent("row-limit", title, 15, optTexts, optValues, description);
    this.className = "RowLimitProp";
  }
});

/* ------------------------------------------------------------------------ */

var WidthLimitProp = ListProp.extend({
  initialize: function() {
    var optTexts = [tr("No limit"), 
                    "20 " + tr("first letters"), "30 " + tr("first letters"), 
                    "35 " + tr("first letters"), "40 " + tr("first letters"), 
                    "50 " + tr("first letters"),  "60 " + tr("first letters"), 
                    "80 " + tr("first letters"), "100 " + tr("first letters")];
    var optValues = [-1, 20, 30, 35, 40, 50, 60, 80, 100];
    this.parent("width-limit", "Text width:", 35, optTexts, optValues,
              "How many chars per line display in Hint?");
    this.className = "WidthLimitProp";
  }
});

/* ------------------------------------------------------------------------ */

// Don't use ListProp here. This version uses always an up-to-date drive list.
var DiskProp = StringProp.extend({
  initialize: function(desc, hideCD) {
    this.parent("disk", "");
    this.className = "DiskProp";
    this.hideCD = (hideCD != null)?hideCD:true;
    this.description = desc ? tr(desc) : null;
  },
  
  getFormField: function(ff) {
    var f = new FormField();
    f.title = tr("Drive:");
    f.type = "popup";
    f.option = [];
    f.optionValue = [];
    var vols = filesystem.volumes;
    for (vol in vols) {
      if (!vols[vol].path) continue;
      if (this.hideCD && (vols[vol].freeBytes == 0)) continue;
      //f.option.push(filesystem.getDisplayName(vols[vol].path) + " - " + vols[vol].path);
      f.option.push(filesystem.getDisplayName(vols[vol].path));
      f.optionValue.push(vols[vol].path);
    }
    f.defaultValue = this.value;
    f.description = this.description;
    ff.push(f);
  }
});

/* ------------------------------------------------------------------------ */
