
//      MiniHTML: mini HTML render class
//      by Andy Chentsov <chentsov@gmail.com> (http://www.extensiblesoft.com).
//
//      Support tags:
//        <div>, <span>, <p>, <br>, <b>, <strong>, <i>, <em>, <u>, <a>, <img>,
//        <cite>, <address>, <sample>, <tt>, <code>, <h1>, <h2>, <h3>, <h4>,
//        <ul>, <ol>, <li>, <dd>, <dl>, <blockquote>, <sub>, <sup>, <font>, <hr>
//
//      The class uses a modified version of mootools.

var MiniHTML = new Class({

  defaults: function() {
    this.css = {
      "text": {color: "#FFFFFF", fontSize: "13px"},
      "p":    {},
      "b":    {fontWeight: "bold"},
      "i":    {fontStyle: "italic"},
      "u":    {textDecoration: "underline"},
      "a":    {color: "#80C0FF", textDecoration: "underline"},
      "h1":   {fontSize: "18px"},
      "h2":   {fontSize: "16px"},
      "h3":   {fontSize: "15px", fontWeight: "bold"},
      "h4":   {fontSize: "14px", fontWeight: "bold"},
      "code": {fontFamily: "monospace", fontWeight: "bold"},
    };
    // CSS aliases
    this.css["strong"] = this.css["b"];
    this.css["em"] = this.css["i"];
    this.css["cite"] = this.css["i"];
    this.css["address"] = this.css["i"];
    this.css["sample"] = this.css["code"];
    this.css["tt"] = this.css["code"];
    
    this.hrefBase = "";
    this.lineImage = "";
    this.loadingImage = "";
  },
  
  initialize: function(frame) {
    this.frame = frame;
    this.fullView = null;
    this.defaults();
  },
  
  prepareXML: function(html) {

    // pick up img tags
    s = html.replace(/<img([^>]*[^\/])>/gi, "<img$1/>");
    
    // pick up br's
    s = s.replace(/<br>/gi, "<br/>");
    
    // pick up hr's
    s = s.replace(/<hr([^>]*[^\/]?)>/gi, "<hr$1/>");
    
    // pull out comments
    s = s.replace(/<!--([^-]|-[^-]|--[^>])*-->/g, "");
    
    // pull out script tags
    s = s.replace(/<script[^>]*>([^<]|<[^\/]|<\/[^s]|<\/s[^c]|<\/sc[^r]|<\/scr[^i]|<\/scri[^p]|<\/scrip[^t]|<\/script[^>])*<\/script>/gi, "");
    
    // pull out style tags
    s = s.replace(/<style[^>]*>([^<]|<[\/]|<\/[^s]|<\/s[^t]|<\/st[^y]|<\/sty[^l]|<\/styl[^e]|<\/style[^>])*<\/style>/gi, "");

    // fix attributes
    s = s.replace(/(\s[a-z]+)=([^\s|"|\/|>]+)/gi, "$1=\"$2\"");

    // strip all around <body>
    var body = s.replace(/\r|\n/ig, "").match(/<body>(.*)<\/body>/ig);
    if (body) s = body.toString();

    // fix DOMXML bug
    s = s.replace(/>\s/gi, ">&#160;");

    // mdash
    // TODO: fix entities
    s = s.replace(/&mdash;/gi, "&#x2014;");

    return "<body>" + s + "</body>";
  },
  
  reset: function() {
    this.styles = [];
    this.currentHOffset = 0;
    this.currentVOffset = 0;
    this.scanHeight = 1;
    this.params = [];
    this.row = [];
    this.indentation = 0;
    this.listStack = [];
    this.textPrefix = "";
    this.baseline = 0;
    this.sizeCorrection = 0;
    this.anchors = [];
  },
  
  render: function(html) {
    this.reset();

    this.xml = this.prepareXML(html);

    //print(this.xml);
    
    try {
      var dom = XMLDOM.parse(this.xml);
      this.render_dom(dom);
    } catch(e) {
      print(e);
    }
  },

  render_dom: function(dom) {
    this.renderNode(dom.documentElement);
    this.alignRow();
  },
  
  renderNode: function(node) {
    if (!node) return;
    switch(node.nodeType) {
      case 1: /* Tag */
        this.renderTag(node);
        break;
        
      case 3: /* Text */
        this.renderText(node);
        break;
    }
    
  },

  renderTag: function(node) {
    var ownStyles = 0;
    var ownParams = 0;
    if (this.css[node.nodeName]) {
      ownStyles++;
      this.styles.push(this.css[node.nodeName]);
    }
    var className = node.getAttribute("class");
    if (className != "") {
      if (this.css["." + className]) {
        ownStyles++;
        this.styles.push(this.css["." + className]);
      }
    }
    switch(node.nodeName) {
      case "h1":
      case "h2":
      case "h3":
      case "h4":
        this.breakLine();
        break;
      case "p":
      case "div":
      case "tr":
      case "blockquote":
        this.breakLine();
        break;
      case "br":
        this.breakLine(false);
        break;
      case "a":
        var href = this.absUrl(node.getAttribute("href"));
        var title = node.getAttribute("title");
        if (title != "") title += "\n";
        var params = {};
        
        if (href != "") {
          if (href.charAt(0) == "#") {
            params = {
              onMouseUp: function() { this.scrollTo(href.substr(1)); }.bind(this),
              tooltip: title + "Scroll to: " + href.substr(1)
            }
          } else {
            params = {
              onMouseUp: function() { openURL(href); },
              tooltip: title + "Open: " + href
            }
          }
        } else {
          this.anchors[node.getAttribute("name").toLowerCase()] = this.currentVOffset - this.scanHeight;
        }
        
        this.params.push(params);
        ownParams++;
        break;
      case "dl":
        this.listStack.push("");
        this.indent();
        break;
      case "ul":
        this.listStack.push("\u2022");
        this.indent();
        break;
      case "ol":
        var type = node.getAttribute("type");
        if (type != "") {
          this.listStack.push(type);
        } else {
          this.listStack.push("1");
        }
        this.indent();
        break;
      case "dd":
        this.breakLine();
        break;
      case "li":
        this.breakLine();
        var bullet = this.listStack[this.listStack.length - 1];
        if (/^\d+$/.test(bullet)) {
          // numbered
          this.textPrefix = bullet + ". ";
          this.listStack[this.listStack.length - 1] = (parseInt(bullet, 10) + 1).toString();
        } else if (/^[a-zA-Z]$/.test(bullet)) {
          // lettered
          this.textPrefix = bullet + ". ";
          if (bullet == "z" || bullet == "Z") {
            bullet = String.fromCharCode(bullet.charCodeAt(0) - 25);
          } else {
            bullet = String.fromCharCode(bullet.charCodeAt(0) + 1);
          }
          this.listStack[this.listStack.length - 1] = bullet;
        } else {
          // other
          this.textPrefix = bullet + " ";
        }
        break;
      case "sup":
        this.sizeCorrection = 0.7;
        this.baseline = this.scanHeight * 0.3;
        break;
      case "sub":
        this.sizeCorrection = 0.7;
        this.baseline = this.scanHeight * -0.3;
        break;
      case "font":
        var style = {};
        for (var i = 0; i < node.attributes.length; i++) {
          var attr = node.attributes.item(i);
          switch(attr.name) {
            case "face":
              style.fontFamily = $font(attr.value);
              break;
            case "size":
              style.fontSize = this.fontSize(attr.value) + "px";
              break;
            case "color":
              style.color = attr.value;
              break;
          }
        }
        this.styles.push(style);
        break;
      case "hr":
        this.renderHLine();
        break;
      case "img":
        this.renderImage(node);
        break;
    }
    
    if (node.hasChildNodes()) {
      for (var i = 0; i < node.childNodes.length; i++) {
        this.renderNode(node.childNodes.item(i));
      }
    }

    while (ownStyles > 0) {
      this.styles.pop();
      ownStyles--;
    }
    while (ownParams > 0) {
      this.params.pop();
      ownParams--;
    }
    switch (node.nodeName) {
      case "h1":
      case "h2":
      case "h3":
      case "h4":
        this.breakLine();
        break;
      case "ol":
      case "ul":
      case "dl":
        this.outdent();
        this.listStack.pop();
        this.breakLine();
        break;
      case "sup":
      case "sub":
        this.baseline = 0;
        this.sizeCorrection = 0;
        break;
    }
  },

  renderHLine: function() {
    if (this.lineImage != "") {
      this.currentVOffset += 14;
      this.breakLine(false);
      var lineImage = new Image();
      lineImage.src = this.lineImage;
      this.frame.appendChild(lineImage);
      lineImage.vAlign = "bottom";
      lineImage.width = this.getWidth();
      lineImage.height = 1;
      lineImage.hOffset = this.indentation;
      lineImage.vOffset = this.currentVOffset - 5;
      this.scanHeight = Math.max(this.scanHeight, lineImage.height);
    } else {
      this.breakLine(false);
      this.currentVOffset += 17;
    }
  },

  renderImage: function(node) {
    var src = node.getAttribute("src");
    if (src == "") return;
    
    var width = node.getAttribute("width");
    var height = node.getAttribute("height");

    this.breakLine(false);
    this.currentVOffset += 7;

    var theImage = new Image();
    var tooltip = node.getAttribute("alt");
    if (tooltip == "") {
      tooltip = node.getAttribute("title");
    }
    if (tooltip != "") {
      theImage.tooltip = tooltip;
    }
    if (width == "") {
      theImage.width = 54;
    }
    if (height == "") {
      theImage.height = 51;
    }
    if ((width == "") || (height == "")) {
      theImage.onImageLoaded = this.imageLoaded.pass(theImage, this);
    }
    theImage.remoteAsync = true;
    if (this.loadingImage != "") {
      theImage.loadingSrc = this.loadingImage;
    }
    theImage.src = src;
    if (theImage.srcWidth > this.frame.width) {
      width = this.frame.width;
      height = Math.round(theImage.srcHeight / (theImage.srcWidth / this.frame.width));
    }
    this.frame.appendChild(theImage);
    if (width != "") {
      theImage.width = width;
    }
    if (height != "") {
      theImage.height = height;
    }
    theImage.hOffset = this.indentation;
    theImage.vOffset = this.currentVOffset - 5;
    this.scanHeight = Math.max(this.scanHeight, theImage.height);
    this.assignParams(theImage);
    
    // TODO: Add flow
  },

  imageLoaded: function(image) {
    // TODO: Add full view hover if bigger than default size
    print(image.src);
    if ((image.width < image.srcWidth) || (image.height < image.srcHeight)) {
      image.onMouseEnter = this.showFullImage.pass(image, this);
      image.onMouseExit = this.hideFullImage.pass(image, this);
    }
  },
  
  showFullImage: function(image) {
    if (!this.fullView) {
      this.fullView = new FullView();
    }
    var p = $absOffset(image);
    this.fullView.show(p.x + image.width, p.y + image.height, image.src);
  },
  
  hideFullImage: function(image) {
    if (this.fullView) {
      this.fullView.hide();
    }
  },

  scrollTo: function(anchor) {
    this.frame.scrollY = -this.anchors[anchor];
  },
  
  splitWords: function(str) {
    var a = [];
    var wordBoundary;
    while ((wordBoundary = str.search(/[\s\n-]/)) >= 0) {
      a.push(str.substr(0, wordBoundary + 1));
      str = str.substr(wordBoundary + 1);
    }
    a.push(str);
    return a;
  },

  getWidth: function() {
    return this.frame.width;
  },

  renderText: function(node) {
    var words = this.splitWords(this.textPrefix + node.data);
    if (words.length == 0) {
      return;
    }
    this.textPrefix = "";

    var text;
    var newLine = true;
    var i = 0;
    do {
      if (newLine) {
        text = new Text();
        text.anchorStyle = "topLeft";
        text.hOffset = this.currentHOffset;
        text.vOffset = this.baseline + this.currentVOffset;
        this.applyStyle(text);
        this.assignParams(text);
        this.frame.appendChild(text);
        newLine = false;
        this.row.push(text);
      }

      var oldData = text.data;
      var oldWidth = text.width;
      
      // Remove leading whitespace if this is the first word on a line
      if (text.hOffset == this.indentation) {
        words[i] = words[i].replace(/^\s+/, "");
      }
      text.data += words[i];
      if (system.platform == "windows") {
        text.data += "\t";
      }

      this.scanHeight = Math.max(text.height, this.scanHeight);
      this.currentHOffset += (text.width - oldWidth);
      
      if ((text.hOffset + text.width) > (this.getWidth())) {
        this.breakLine();
        newLine = true;
        // wrap this word to the next line
        text.data = oldData;
      } else {
        i++;
      }

    } while(i < words.length);
  },

  assignParams: function(element) {
    if (!element) return;
    if (this.params.length > 0) {
      var params = this.params[this.params.length - 1];
      for (var name in params) {
        if ((typeof(params[name]) == "function") &&
            (name.substr(0, 2).toLowerCase() != "on")) continue;
        element[name] = params[name];
      }
    }
  },

  breakLine: function() {
    this.alignRow();
    this.row = [];
    this.currentHOffset = this.indentation;
    this.currentVOffset += this.scanHeight;
    if (arguments.length == 0) {
      this.scanHeight = 1;
    }
  },

  indent: function() {
    this.indentation = Math.min(this.getWidth() / 2, this.indentation + 10);
    this.currentHOffset = Math.max(this.indentation, this.currentHOffset);
  },

  outdent: function() {
    this.indentation = Math.max(0, this.indentation - 10);
    this.currentHOffset = Math.max(this.indentation, this.currentHOffset - 10);
  },

  alignRow: function() {
    if (this.row.length == 0) return;
    // find max height
    var maxHeight = 0;
    for (var i = 0; i < this.row.length; i++) {
      maxHeight = Math.max(maxHeight, this.row[i].height);
    }
    
    for (var i = 0; i < this.row.length; i++) {
      if (this.row[i].height < maxHeight) {
        this.row[i].vOffset += maxHeight - this.row[i].height;
      }
    }
  },

  getCurrentStyle: function() {
    var result = {};
    if (this.css["text"]) {
      result = this.css["text"].copy();
    }
    for(var i = 0; i < this.styles.length; i++) {
      for(var j in this.styles[i]) {
        if (typeof(this.styles[i][j]) == "function") continue;
        result[j] = this.styles[i][j];
      }
    }
    return result;
  },

  applyStyle: function(element) {
    var style = this.getCurrentStyle();
    for(var i in style) {
      if (typeof(style[i]) == "function") continue;
      value = style[i];
      if ((i == "fontSize") && (this.sizeCorrection != 0)) {
        value = (parseInt(value.replace("px")) * this.sizeCorrection) + "px";
      }
      element.style[i] = value;
    }
  },

  absUrl: function(url) {
    if (/[^\/]*:\/\//.test(url)) {
      // absolute
      return url;
    } else {
      // relative
      if (url.charAt(0) == "/") {
        var result = this.hrefBase.match(/^([^:]*:\/\/[^\/]*)/);
        if (result) {
          return result[1] + url;
        } else {
          return this.hrefBase + url;
        }
      } else {
        return url;
      }
    }
  },

  fontSize: function(size) {
    if (typeof size == "string") {
      size = parseFloat(size);
    }
    result = size;
    switch(size) {
      case 1:
        result = 9;
        break;
      case 2:
        result = 13;
        break;
      case 3:
        result = 15;
        break;
      case 4:
        result = 18;
        break;
      case 5:
        result = 20;
        break;
    }
    if (result < 5) {
      result = 5;
    }
    return result;    
  }

});

var FullView = new Class({
  
  initialize: function() {
    this.win = new Window();
    this.win.level = "floating";
    this.image = new Image();
    this.image.window = this.win;
  },
  
  show: function(x, y, src) {
    this.image.src = src;
    this.win.width = this.image.width;    
    this.win.height = this.image.height;
    
    // TODO: fix coords
    
    this.win.hOffset = x;
    this.win.vOffset = y;
    if (!this.win.visible) {
      this.win.opacity = 0;
      this.win.visible = true;    
      this.win.opacity = 255;
    }
  },
  
  hide: function() {
    this.win.opacity = 0;
    this.win.visible = false;
  }
  
});
